package nl.amis.view;

import java.lang.reflect.Field;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.el.ELContext;
import javax.el.ExpressionFactory;

import javax.el.ValueExpression;

import javax.faces.application.Application;
import javax.faces.context.FacesContext;

import javax.faces.event.ActionEvent;

import oracle.adf.share.logging.ADFLogger;

import sun.misc.SoftCache;

public class MessageManager {

    Object contextDimensionValue;
    private static final ADFLogger sLog =
        ADFLogger.createADFLogger(MessageManager.class);

    List<String> bundleNames = new ArrayList<String>();
    List<String> contextDimensionValues = new ArrayList<String>();

    public String getMessage(String key) {

        String msg = getMessageFromResourceBundle(key, (String)contextDimensionValue);
        if (msg == null || msg.length() == 0 || msg.startsWith("???"))
            msg = getMessageFromResourceBundle(key, "");
        return msg;
    }

    private String getMessageFromResourceBundle(String key,
                                                String bundlePostfix) {
        ResourceBundle bundle = null;
        String message = "";
        Locale locale =
            FacesContext.getCurrentInstance().getViewRoot().getLocale();
        System.out.println("key for locale "+locale.getLanguage());
        for (String resourcebundle : bundleNames) {
            String bundleName =
                resourcebundle + (bundlePostfix == null ? "" : bundlePostfix);
            try {
                bundle =
                        ResourceBundle.getBundle(bundleName, locale, getCurrentLoader(bundleName));
            } catch (MissingResourceException e) {
                // bundle with this name not found;
            }
            if (bundle == null)
                continue;
            try {
                message = bundle.getString(key);
                if (message != null)
                    break;
            } catch (Exception e) {
            }
        } // loop over bundles
        return message;

    }


    private String getMessageFromJSFBundle(String key) {
        return (String)resolveExpression("#{msgbundle['" + key + "']}");
    }

    private String getMessageFromPrefixedBundle(String key,
                                                String bundlePrefix) {
        return (String)resolveExpression("#{" + bundlePrefix + "msgbundle['" +
                                         key + "']}");
    }


    public static ClassLoader getCurrentLoader(Object fallbackClass) {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        if (loader == null)
            loader = fallbackClass.getClass().getClassLoader();
        return loader;
    }


    // from JSFUtils in Oracle ADF 11g Storefront Demo

    public static Object resolveExpression(String expression) {
        FacesContext facesContext = FacesContext.getCurrentInstance();
        Application app = facesContext.getApplication();
        ExpressionFactory elFactory = app.getExpressionFactory();
        ELContext elContext = facesContext.getELContext();
        ValueExpression valueExp =
            elFactory.createValueExpression(elContext, expression,
                                            Object.class);
        return valueExp.getValue(elContext);
    }

    public void actionForceBundleRefresh(ActionEvent ae) {
        forceBundleRefresh();
    }

    /**
     * Refresh all resource bundle returned by getBundleNames() in the JVM cache
     */
    public void forceBundleRefresh() {
        // this clearCache mechanism only works as of jdk 1.6
        try {
            ResourceBundle.clearCache();
            sLog.info("Resourcebundle.clearCache executed succesfully.");
        } catch (Exception e) {
            sLog.info("Resourcebundle.clearCache cannot be used because jdk version is not 1.6 or higher, try old approach");
            // try old approach
            for (String resourcebundle : bundleNames) {
                for (String contextDimensionValue : contextDimensionValues) {
                    String bundleName = resourcebundle + contextDimensionValue;
                    refreshBundle(bundleName);
                }
            } //for
        }
    }


    /**
     * Refresh a named resource bundle in the JVM cache.
     * @param bundleName
     */
    protected void refreshBundle(String bundleName) {
        try {
            SoftCache cache =
                (SoftCache)getFieldFromClass(ResourceBundle.class,
                                             "cacheList");

            synchronized (cache) {
                ArrayList myBundles = new ArrayList();

                Iterator keyIter = cache.keySet().iterator();
                while (keyIter.hasNext()) {
                    Object key = keyIter.next();
                    String name =
                        (String)getFieldFromObject(key, "searchName");

                    if (name.startsWith(bundleName)) {
                        myBundles.add(key);
                        sLog.info("Resourcebundle " + name +
                                  " will be refreshed.");
                    }
                }
                cache.keySet().removeAll(myBundles);
            }
        } catch (Exception e) {
            sLog.severe("Failed to refresh bundle: " + e.getMessage());
        }
    }

    private Object getFieldFromObject(Object object,
                                      String fieldName) throws NoSuchFieldException,
                                                               IllegalAccessException {
        return getFieldValue(object.getClass(), object, fieldName);
    }

    private Object getFieldFromClass(Class clazz,
                                     String fieldName) throws NoSuchFieldException,
                                                              IllegalAccessException {
        return getFieldValue(clazz, null, fieldName);
    }

    private Object getFieldValue(Class clazz, Object obj,
                                 String fieldName) throws NoSuchFieldException,
                                                          IllegalAccessException {
        Field field = clazz.getDeclaredField(fieldName);

        boolean accessible = field.isAccessible();
        field.setAccessible(true);

        try {
            return field.get(obj);
        } finally {
            field.setAccessible(accessible);
        }
    }

    public void setBundleNames(List<String> bundleNames) {
        this.bundleNames = bundleNames;
    }

    public List<String> getBundleNames() {
        return bundleNames;
    }

    public void setcontextDimensionValues(List<String> contextDimensionValues) {
        this.contextDimensionValues = contextDimensionValues;
    }

    public List<String> getcontextDimensionValues() {
        return contextDimensionValues;
    }

    public void setContextDimensionValue(Object contextDimensionValue) {
        this.contextDimensionValue = contextDimensionValue;
    }

    public Object getContextDimensionValue() {
        return contextDimensionValue;
    }
}
